/// <reference types="../types/chai" />
/// <reference types="../types/mocha" />
declare module 'creevey/lib/types' {
	/// <reference types="node" />
	import { Worker as ClusterWorker } from 'cluster';
	import { Context } from 'mocha';
	export interface Capabilities {
	    browserName: string;
	    resolution?: {
	        width: number;
	        height: number;
	    };
	}
	export interface BrowserConfig {
	    limit: number;
	    gridUrl?: string;
	    address?: Address;
	    testRegex?: RegExp;
	}
	interface Address {
	    host: string;
	    port: number;
	    path: string;
	}
	export interface Config {
	    gridUrl: string;
	    address: Address;
	    testRegex: RegExp;
	    testDir: string;
	    screenDir: string;
	    reportDir: string;
	    maxRetries: number;
	    threshold: number;
	    browsers: {
	        [key: string]: Capabilities & BrowserConfig;
	    };
	    hooks: {
	        beforeAll: (this: Context) => void;
	        beforeEach: (this: Context) => void;
	    };
	}
	export interface Options {
	    config: string;
	    parser: boolean;
	    ui: boolean;
	    browser?: string;
	    reporter?: string;
	}
	export interface Worker extends ClusterWorker {
	    isRunnning?: boolean;
	}
	export type WorkerMessage = {
	    type: "ready";
	} | {
	    type: "error";
	    payload: any;
	} | {
	    type: "test";
	    payload: TestResult;
	};
	export interface Images {
	    actual: string;
	    expect?: string;
	    diff?: string;
	}
	export type TestStatus = "pending" | "running" | "failed" | "success";
	export interface TestResult {
	    status: "failed" | "success";
	    images?: Partial<{
	        [name: string]: Images;
	    }>;
	    error?: string;
	}
	export interface Test {
	    id: string;
	    path: string[];
	    skip: boolean;
	    status?: TestStatus;
	    results?: TestResult[];
	    approved?: Partial<{
	        [image: string]: number;
	    }>;
	}
	export interface CreeveyStatus {
	    isRunning: boolean;
	    testsById: Partial<{
	        [id: string]: Test;
	    }>;
	}
	export interface CreeveyUpdate {
	    isRunning?: boolean;
	    testsById?: Partial<{
	        [id: string]: Partial<Test>;
	    }>;
	}
	export interface ApprovePayload {
	    id: string;
	    retry: number;
	    image: string;
	}
	export type Request = {
	    type: "start";
	    payload: string[];
	} | {
	    type: "stop";
	} | {
	    type: "approve";
	    payload: ApprovePayload;
	};
	export type Response = {
	    type: "status";
	    seq: number;
	    payload: CreeveyStatus;
	} | {
	    type: "update";
	    seq: number;
	    payload: CreeveyUpdate;
	};
	export function isTest<T1, T2 extends Test>(x: T1 | T2): x is T2;
	export function isDefined<T>(value: T | undefined): value is T;
	export {};

}
declare module 'creevey/lib/utils' {
	import { Context } from 'mocha';
	import { Config } from 'creevey/lib/types';
	export function getBrowser(config: Config, browserName: string): Promise<import("selenium-webdriver").WebDriver>;
	export function switchStory(this: Context): Promise<void>;

}
declare module 'creevey/lib/config' {
	import { Config } from 'creevey/lib/types';
	export const defaultConfig: {
	    testRegex: string;
	    testDir: string;
	    screenDir: string;
	    reportDir: string;
	    maxRetries: number;
	    threshold: number;
	};
	export function readConfig(configPath: string): Config;

}
declare module 'creevey/lib/server/index' {
	import { Options } from 'creevey/lib/types';
	export default function (options: Options): Promise<void>;

}
declare module 'creevey/lib/loader' {
	export class Loader {
	    private testRegex;
	    private loadCallback;
	    constructor(testRegex: RegExp, loadCallback: (filePath: string) => void);
	    loadTests(directory: string): Promise<void>;
	}

}
declare module 'creevey/lib/mocha-ui/chai-image' {
	/// <reference types="chai" />
	import { Config } from 'creevey/lib/types'; const _default: (config: Config, context: string[], onSaveImage?: (imageName: string, imageNumber: number, type: "actual" | "expect" | "diff") => void) => ({ Assertion }: any, utils: Chai.ChaiUtils) => void;
	export default _default;

}
declare module 'creevey/lib/mocha-ui/helpers' {
	import { Suite, TestFunction, SuiteFunction } from 'mocha';
	import { CommonFunctions, CreateOptions } from 'mocha/lib/interfaces/common';
	import { Config } from 'creevey/lib/types'; type CreateSuite = (options: CreateOptions, parentSuite: Suite) => Suite; type Describer = (title: string, fn: (this: Suite) => void, createSuite: CreateSuite) => Suite | Suite[];
	export function createBrowserSuites(config: Config, suites: Suite[]): Suite[];
	export function createDescriber(config: Config, browserSuites: Suite[], suites: Suite[], file: string): Describer;
	export function describeFactory(describer: Describer, common: CommonFunctions): SuiteFunction;
	export function itFactory(suites: Suite[], file: string, common: CommonFunctions): TestFunction;
	export {};

}
declare module 'creevey/lib/mocha-ui/ui' {
	export {};

}
declare module 'creevey/lib/mocha-ui/index' {
	import 'creevey/lib/mocha-ui/ui';

}
declare module 'creevey/lib/server/master/pool' {
	/// <reference types="node" />
	import { EventEmitter } from 'events';
	import { Config } from 'creevey/lib/types';
	export default class Pool extends EventEmitter {
	    private maxRetries;
	    private browser;
	    private config;
	    private workers;
	    private queue;
	    private forcedStop;
	    readonly isRunning: boolean;
	    constructor(config: Config, browser: string);
	    init(): Promise<unknown[]>;
	    start(tests: {
	        id: string;
	        path: string[];
	    }[]): boolean;
	    stop(): void;
	    process(): void;
	    private sendStatus;
	    private getFreeWorker;
	    private readonly freeWorkers;
	    private handleMessage;
	}

}
declare module 'creevey/lib/server/master/runner' {
	/// <reference types="node" />
	import { EventEmitter } from 'events';
	import { Config, Test, CreeveyStatus, ApprovePayload } from 'creevey/lib/types';
	export default class Runner extends EventEmitter {
	    private screenDir;
	    private reportDir;
	    private tests;
	    private browsers;
	    private pools;
	    readonly isRunning: boolean;
	    constructor(config: Config, tests: Partial<{
	        [id: string]: Test;
	    }>);
	    private handlePoolMessage;
	    private handlePoolStop;
	    init(): Promise<unknown[][]>;
	    start(ids: string[]): void;
	    stop(): void;
	    readonly status: CreeveyStatus;
	    approve({ id, retry, image }: ApprovePayload): Promise<void>;
	    private sendUpdate;
	}

}
declare module 'creevey/lib/server/master/api' {
	import WebSocket from 'creevey/ws';
	import Runner from 'creevey/lib/server/master/runner';
	export interface CreeveyApi {
	    subscribe: (wss: WebSocket.Server) => void;
	    handleMessage: (message: WebSocket.Data) => void;
	}
	export default function creeveyApi(runner: Runner): CreeveyApi;

}
declare module 'creevey/lib/server/master/master' {
	import { Config } from 'creevey/lib/types';
	import Runner from 'creevey/lib/server/master/runner';
	export default function master(config: Config): Promise<Runner>;

}
declare module 'creevey/lib/server/master/server' {
	import { CreeveyApi } from 'creevey/lib/server/master/api';
	export default function server(api: CreeveyApi, reportDir: string): void;

}
declare module 'creevey/lib/server/master/index' {
	import { Config, Options } from 'creevey/lib/types';
	export default function (config: Config, options: Options): Promise<void>;

}
declare module 'creevey/lib/server/master/parser' {
	import { Config } from 'creevey/lib/types';
	export default function parse(config: Config): Promise<void>;

}
declare module 'creevey/lib/server/worker/reporter' {
	import { Runner, reporters, MochaOptions } from 'mocha';
	export class CreeveyReporter extends reporters.Base {
	    constructor(runner: Runner, options: MochaOptions);
	}
	export class TeamcityReporter extends reporters.Base {
	    constructor(runner: Runner, options: MochaOptions);
	    private escape;
	}

}
declare module 'creevey/lib/server/worker/worker' {
	import { Config, Options } from 'creevey/lib/types';
	export default function worker(config: Config, options: Options & {
	    browser: string;
	}): Promise<void>;

}
declare module 'creevey/lib/server/worker/index' {
	export { default } from 'creevey/lib/server/worker/worker';

}
declare namespace Chai {
  interface ChaiStatic {
    use(fn: (chai: any, utils: ChaiUtils) => void): ChaiStatic;
  }

  interface Assertion {
    matchImage: (filename: string) => void;
  }
}
declare namespace Mocha {
  // NOTE @types/mocha don't have `retries` method in MochaGlobals
  interface MochaGlobals {
    retries: (n: number) => MochaGlobals;
  }

  interface Context {
    config: import("creevey/lib/types").Config;
    browser: import("selenium-webdriver").WebDriver;
    browserName: string;
    testScope: string[];
  }

  interface SuiteFunction {
    (title: string, fn: (this: Suite) => void): Suite | Suite[];
    only: ExclusiveSuiteFunction;
    skip: PendingSuiteFunction;
  }

  interface ExclusiveSuiteFunction {
    (browsers: string[], title: string, fn: (this: Suite) => void): Suite | Suite[];
  }

  interface PendingSuiteFunction {
    (browsers: string[], title: string, fn: (this: Suite) => void): Suite | Suite[];
  }

  interface TestFunction {
    only: ExclusiveTestFunction;
    skip: PendingTestFunction;
  }

  interface ExclusiveTestFunction {
    (browsers: string[], title: string, fn?: AsyncFunc): Test;
  }

  interface PendingTestFunction {
    (browsers: string[], title: string, fn?: AsyncFunc): Test;
  }
}
